
from django.contrib import admin

from models import LastUsed


class LastUsedAdmin(admin.ModelAdmin):

    search_fields = ('user__username', 'content_type__name')
    list_filter = ('last_used', 'key', 'content_type')

    list_display = ('user', 'content_type', 'content_object', 'last_used')


admin.site.register(LastUsed, LastUsedAdmin)




from models import LastUsed, ContentType


LIMIT = 10


def use(object, user, key=None, when=None):
    args = {
       'user': user,
       'content_object': object
    }

    if when:
        args['last_used'] = when

    if key:
        args['key'] = key

    obj = LastUsed.objects.create(**args)

    filters = args.copy()

    del filters['content_object']
    filters['content_type'] = obj.content_type

    for lu in LastUsed.objects.filter(**filters)[LIMIT:]:
        lu.delete()

    return obj


def get(model=None, user=None, key=None, limit=0):

    filters = {}
    if model:
        filters['content_type'] = ContentType.objects.get_for_model(model)

    if user:
        filters['user'] = user

    if key:
        filters['key'] = key

    objects = LastUsed.objects.filter(**filters)

    if limit:
        objects = objects[:limit]

    return objects




from django.db import models
from django.contrib.auth.models import User
from django.utils import timezone
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic


class LastUsed(models.Model):
    user = models.ForeignKey(User)
    last_used = models.DateTimeField(default=timezone.now)

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    key = models.CharField(max_length=64, blank=True, null=True)

    class Meta:
        ordering = ("-last_used",)

    def __unicode__(self):
        if self.key:
            return u"%s used %s (%s) on %s" % \
                    (self.user, self.content_object, self.key, self.last_used)
        return u"%s used %s on %s" % \
                    (self.user, self.content_object, self.last_used)



"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""
import datetime
from django.test import TestCase
from django.utils.timezone import utc

from last_used import use, get, LIMIT
from last_used.models import LastUsed, User


class TestUse(TestCase):

    def setUp(self):
        self.user = User.objects.create(username='test_user')

    def test_use(self):

        obj = use(self.user, self.user)

        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.content_object, self.user)

    def test_use_key(self):

        obj = use(self.user, self.user, key="some_key")

        self.assertEqual(obj.key, "some_key")
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.content_object, self.user)

    def test_use_when(self):
        when = datetime.datetime(2000, 1, 1, 4, 3, 2, 1).replace(tzinfo=utc)
        obj = use(self.user, self.user, when=when)

        self.assertEqual(obj.last_used, when)
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.content_object, self.user)

    def test_use_delete(self):

        for i in range(LIMIT + 2):
            use(self.user, self.user)

        self.assertEqual(LastUsed.objects.count(), LIMIT)

    def test_use_keys(self):

        for i in range(LIMIT + 2):
            use(self.user, self.user, key=str(i))

        self.assertEqual(LastUsed.objects.count(), LIMIT + 2)


class TestGet(TestCase):

    def setUp(self):
        self.user = User.objects.create(username='test_user')

    def test_empty(self):

        objs = get(self.user, self.user)

        self.assertEqual(objs.count(), 0)

    def test_found(self):

        last_used = LastUsed.objects.create(user=self.user,
                                        content_object=self.user)
        objs = list(get(User, self.user))

        self.assertEqual(len(objs), 1)
        self.assertEqual(objs[0], last_used)

    def test_limit(self):

        for i in range(4):
            LastUsed.objects.create(user=self.user, content_object=self.user)

        objs = list(get(self.user, self.user, limit=2))

        self.assertEqual(len(objs), 2)

    def test_no_model(self):

        objs = get(user=self.user)

        self.assertEqual(objs.count(), 0)

    def test_no_user(self):

        objs = get(model=self.user)

        self.assertEqual(objs.count(), 0)

    def test_with_key(self):

        objs = get(key="some_key")

        self.assertEqual(objs.count(), 0)


class TestModel(TestCase):

    def test_unicode_no_key(self):
        user = User(username="test_user")
        last_used = LastUsed(content_object=user, user=user)

        self.assertIn(unicode(user), unicode(last_used))

    def test_unicode_key(self):
        user = User(username="test_user")
        last_used = LastUsed(content_object=user, user=user, key="key_test")

        self.assertIn(unicode(user), unicode(last_used))
        self.assertIn("key_test", unicode(last_used))



#!/usr/bin/env python

from distutils.core import setup

__author__ = u'Ferran Pegueroles'
__copyright__ = u'Copyright 2013, Ferran Pegueroles'
__credits__ = [u'Ferran Pegueroles']


__license__ = 'GPL'
__version__ = '0.1'
__email__ = 'ferran@pegueroles.com'


long_description = open('README.rst').read()


setup(
    name='django-last-used',
    version=__version__,
    url='http://bitbucket.org/ferranp/django-last-used',
    author=__author__,
    author_email=__email__,
    license='GPL',
    packages=['last_used', 'last_used.templatetags'],
    description='Keep track of last used objects on a django app',
    long_description=long_description,
    classifiers=['Development Status :: 5 - Production/Stable',
                 'Environment :: Web Environment',
                 'Framework :: Django',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: GNU General Public License (GPL)',
                 'Topic :: Internet :: WWW/HTTP :: Dynamic Content']
)
